@@ -12,7 +12,7 @@ use Bio::Gonzales::Util::File qw/open_on_demand is_newer/;
 use Data::Dumper;
 use Cwd qw/fastcwd/;
 
-our $VERSION = '0.0387_01'; # VERSION
+our $VERSION = 0.01_01;
 
 has 'sep' => ( is => 'rw', required => 1, isa => 'Str' );
 has 'sep_pos' => ( is => 'rw', default => '^' );
@@ -69,14 +69,13 @@ sub create {
 
   for my $f (@$abs_input_files) {
 
-    # start of file is the first (chunk) element
     my @file_idx  = (0);
     my $num_elems = 1;
-    my $chunk_elem_count = 1;
 
+    my $chunk_elem_count = 1;
     open my $fh, '<', $f or confess "Can't open filehandle $f: $!";
     unless (<$fh>) {
-      $fh->close;
+      close $fh;
       next;
     }
     while (<$fh>) {
@@ -99,7 +98,7 @@ sub create {
       age       => ( stat $f )[9]
       };
 
-    $fh->close;
+    close $fh;
   }
 
   $self->_internal_info()->{sep} = $self->sep;
