@@ -8,70 +8,40 @@ use Carp;
 use File::Slurp qw(:std);
 use File::Spec::Functions qw/catfile/;
 use Bio::Gonzales::Util::File qw/slurpc open_on_demand/;
+use Mail::Sendmail;
 use Bio::Grid::Run::SGE::Util qw/my_glob MSG/;
 use Bio::Grid::Run::SGE::Log::Worker;
-use Bio::Grid::Run::SGE::Log::Notify::Jabber;
-use Bio::Grid::Run::SGE::Log::Notify::Mail;
-use Bio::Grid::Run::SGE::Config;
-use Bio::Gonzales::Util::Cerial;
 
-use Sys::Hostname;
+our $VERSION = 0.01_01;
 
-our $VERSION = '0.0387_01'; # VERSION
+has config_file    => ( is => 'rw', required   => 1 );
+has c              => ( is => 'rw', required   => 1 );
+has error_cmd_file => ( is => 'rw', lazy_build => 1 );
+has error_log_file => ( is => 'rw', lazy_build => 1 );
+has _log_report    => ( is => 'rw', default    => sub { [] } );
+has _cmd_script    => ( is => 'rw', default    => sub { [] } );
 
-has config_file         => ( is => 'rw', required   => 1 );
-has c                   => ( is => 'rw', required   => 1 );
-has failed_restart_file => ( is => 'rw', lazy_build => 1 );
-has failed_log_file     => ( is => 'rw', lazy_build => 1 );
-has failed_update_file  => ( is => 'rw', lazy_build => 1 );
-has attempts            => ( is => 'rw', lazy_build => 1 );
-has _log                => ( is => 'rw', default    => sub { [] } );
-has _node_log           => ( is => 'rw', default    => sub { [] } );
-has _cmd_script         => ( is => 'rw', default    => sub { [] } );
-has num_jobs            => ( is => 'rw', default    => 0 );
-has num_jobs_skipped            => ( is => 'rw', default    => 0 );
-has num_jobs_failed     => ( is => 'rw', default    => 0 );
-
-sub _build_failed_update_file {
+sub _build_error_log_file {
   my ($self) = @_;
 
   my ( $tmp_dir, $job_name, $job_id ) = @{ $self->c }{qw/tmp_dir job_name job_id/};
-  my $update_log_file = catfile( $tmp_dir, "update.$job_name.j$job_id.sh" );
-  return $update_log_file;
+  my $error_log_file = catfile( $tmp_dir, "$job_name.j$job_id.log" );
+
+  return $error_log_file;
 }
 
-sub _build_failed_log_file {
+sub _build_error_cmd_file {
   my ($self) = @_;
 
   my ( $tmp_dir, $job_name, $job_id ) = @{ $self->c }{qw/tmp_dir job_name job_id/};
-  my $failed_log_file = catfile( $tmp_dir, "log.$job_name.j$job_id.txt" );
+  my $error_cmd_file = catfile( $tmp_dir, "restart.$job_name.j$job_id.sh" );
 
-  return $failed_log_file;
-}
-
-sub _build_failed_restart_file {
-  my ($self) = @_;
-
-  my ( $tmp_dir, $job_name, $job_id ) = @{ $self->c }{qw/tmp_dir job_name job_id/};
-  my $failed_restart_file = catfile( $tmp_dir, "restart.$job_name.j$job_id.sh" );
-
-  return $failed_restart_file;
-}
-
-sub _build_attempts {
-  my ($self) = @_;
-  return $self->c->{notification_attempts} // 3;
+  return $error_cmd_file;
 }
 
 sub _report_log {
   my $self = shift;
-  push @{ $self->_log }, @_;
-  return;
-}
-
-sub _report_node_log {
-  my $self = shift;
-  push @{ $self->_node_log }, @_;
+  push @{ $self->_log_report }, @_;
   return;
 }
 
@@ -94,16 +64,8 @@ sub analyse {
   my $log_dir = my_glob( $c->{log_dir} );
 
   my $something_crashed;
-  my $num_skipped = 0;
 
-  $self->_report_log( 'main log file: ' . $self->failed_log_file, '' );
-
-  $self->_report_log( 'restart failed jobs: ' . $self->failed_restart_file );
-  $self->_report_log( 'update job states: ' . $self->failed_update_file, '' );
-
-  $self->_report_log( 'working dir: ' . $c->{working_dir} );
-  $self->_report_log( 'result dir: ' . $c->{result_dir} );
-  $self->_report_log( 'tmp dir: ' . $c->{tmp_dir}, '' );
+  $self->_report_log( 'working_dir is ' . $c->{working_dir} );
 
   my %jobs_with_log;
 
@@ -171,63 +133,46 @@ sub analyse {
       );
       # track which nodes broke, often one specific node always breaks
       $err_hosts{ $log_data->{hostname} }++;
-    } elsif ( exists( $log_data->{'comp.task.exit.skip'} ) ) {
-      $num_skipped++;
     }
   }
 
   my $no_jobs_ran_at_all;
-  my $num_jobs = 'n/a';
+  my $total_jobs = 'n/a';
   if ( exists( $c->{range} ) ) {
-    $num_jobs = $c->{range}[1] - $c->{range}[0] + 1;
+    $total_jobs = $c->{range}[1] - $c->{range}[0];
   MISSING_JOBS:
     for ( my $i = $c->{range}[0]; $i <= $c->{range}[1]; $i++ ) {
       unless ( exists( $jobs_with_log{$i} ) ) {
         $something_crashed++;
-        if ( $STD_JOB_CMD && $STD_WORKER_WD ) {
-          $self->_report_missing_job(
-            {
-              job_cmd => $STD_JOB_CMD,
-              job_id  => $job_id,
-              id      => $i,
-              cwd     => $STD_WORKER_WD,
-            }
-          );
-          $err_hosts{NA}++;
-        } else {
-          $no_jobs_ran_at_all = 1;
-          $err_hosts{NA} = $num_jobs;
-          last MISSING_JOBS;
-        }
+        $no_jobs_ran_at_all = $self->_report_missing_job(
+          {
+            job_cmd => $STD_JOB_CMD,
+            job_id  => $job_id,
+            id      => $i,
+            cwd     => $STD_WORKER_WD,
+          }
+        );
+        last MISSING_JOBS if ($no_jobs_ran_at_all);
       }
     }
   }
 
-  $self->num_jobs($num_jobs);
-  $self->num_jobs_skipped($num_skipped);
-
-  $self->_report_log(($num_jobs || 'n/a') . " jobs in total");
-  $self->_report_log('(' .$num_skipped . " jobs did not need to run again)") if($num_skipped);
-  if ($no_jobs_ran_at_all) {
-    $self->_report_log("obviously, no jobs were run at all");
-    $self->num_jobs_failed($num_jobs);
-  } elsif ($something_crashed) {
+  $self->_report_log("obviously, no jobs were run at all") if ($no_jobs_ran_at_all);
+  if ($something_crashed) {
     # create a log entry with the hostnames and the number of times crashed.
     # often one node has all crashes and with this you can spot it easily in the log file.
     my @err_host_names = keys %err_hosts;
 
     @err_host_names = sort { $err_hosts{$b} <=> $err_hosts{$a} } @err_host_names;
-    my @log_entries     = ("Failed hosts:");
-    my $num_jobs_failed = 0;
+    my @log_entries  = ("Failed hosts:");
+    my $total_failed = 0;
     for my $h (@err_host_names) {
-      $num_jobs_failed += $err_hosts{$h};
+      $total_failed += $err_hosts{$h};
       push @log_entries, sprintf( "   %3d %s", $err_hosts{$h}, $h );
     }
+    push @log_entries, sprintf( "TOTAL FAILED: %s of %s", $total_failed, $total_jobs );
 
-    $self->_report_log( sprintf( "Jobs failed: %s of %s", $num_jobs_failed, $num_jobs ) );
-    $self->num_jobs_failed($num_jobs_failed);
-
-    $self->_report_log(@log_entries);
+    $self->_report_log( join( "\n", @log_entries ) );
   } else {
     $self->_report_log('all nodes finished successfully');
   }
@@ -243,10 +188,10 @@ sub _report_crashed_job {
 
   $self->_report_cmd(
     "cd '$log_data->{cwd}' && $s->{job_cmd} --range $s->{range} --job_id $s->{job_id} --id $log_data->{id}");
-  $self->_report_node_log( "Node " . $log_data->{id} . " crashed" );
-  $self->_report_node_log("    log: $s->{log_file}");
-  $self->_report_node_log("    err: $s->{err_file}");
-  $self->_report_node_log("    out: $s->{out_file}");
+  $self->_report_log( "Node " . $log_data->{id} . " crashed" );
+  $self->_report_log("    log: $s->{log_file}");
+  $self->_report_log("    err: $s->{err_file}");
+  $self->_report_log("    out: $s->{out_file}");
 
   return;
 }
@@ -254,11 +199,14 @@ sub _report_crashed_job {
 sub _report_missing_job {
   my ( $self, $s ) = @_;
 
+  unless ( $s->{job_cmd} && $s->{cwd} ) {
+    return 1;
+  }
   $self->_report_cmd("#NODE: $s->{id}; NO_LOG");
   #replace job array numbers with worker id, to emulate the environment of the original array job
 
   $self->_report_cmd("cd '$s->{cwd}' && $s->{job_cmd} --job_id $s->{job_id} --id $s->{id}");
-  $self->_report_node_log( "Node " . $s->{id} . " crashed, NO_LOG NO_ERR NO_OUT" );
+  $self->_report_log( "Node " . $s->{id} . " crashed, NO_LOG NO_ERR NO_OUT" );
 
   return;
 }
@@ -273,117 +221,55 @@ sub _report_error_job {
 
     $self->_report_cmd(
       "cd '$log_data->{cwd}' && $s->{job_cmd} --range $task_id --job_id $s->{job_id} --id $log_data->{id}");
-    $self->_report_node_log( "Node " . $log_data->{id} . " had error(s)" );
-    $self->_report_node_log("    log: $s->{log_file}");
-    $self->_report_node_log("    err: $s->{err_file}");
-    $self->_report_node_log("    out: $s->{out_file}");
+    $self->_report_log( "Node " . $log_data->{id} . " had error(s)" );
+    $self->_report_log("    log: $s->{log_file}");
+    $self->_report_log("    err: $s->{err_file}");
+    $self->_report_log("    out: $s->{out_file}");
   }
   return;
 }
 
-sub notify {
+sub send_mail {
   my ($self) = @_;
   my $c = $self->c;
+  return unless ( $c->{mail} );
 
-  # reread config and merge it to get the passwords for the notify stuff
-  $c = { %{ Bio::Grid::Run::SGE::Config->new->config }, %$c };
-  return unless ( $c->{notify} );
+  unshift @{ $Mail::Sendmail::mailcfg{'smtp'} }, $c->{smtp_server} if ( $c->{smtp_server} );
 
-  my %info = (
-    subject => '[LOG]' . $self->subject(),
-    message => $self->gen_report,
-    from    => $self->from_address(),
+  MSG("Sending mail to $c->{mail}.");
+
+  my %mail = (
+    To      => $c->{mail},
+    Subject => __PACKAGE__ . ' ' . $c->{job_name} . " - " . $c->{job_id},
+    From    => (
+      $ENV{SGE_O_LOGNAME} && $ENV{SGE_O_HOST}
+      ? join( '@', $ENV{SGE_O_LOGNAME}, $ENV{SGE_O_HOST} )
+      : join( '@', $ENV{USER},          $ENV{HOSTNAME} )
+    ),
+    Message => join( "\n", @{ $self->_log_report } ),
   );
 
-  if ( $c->{notify}{mail} ) {
-    my $n = Bio::Grid::Run::SGE::Log::Notify::Mail->new( $c->{notify}{mail} );
-    for ( my $i = 0; $i < $self->attempts; $i++ ) {
-      # notify function returns 1 on error. If this happens, try more times
-      last unless ( $n->notify( \%info ) );
-    }
-  }
-  if ( $c->{notify}{jabber} ) {
-    my $n = Bio::Grid::Run::SGE::Log::Notify::Jabber->new( $c->{notify}{jabber} );
-    for ( my $i = 0; $i < $self->attempts; $i++ ) {
-      # notify function returns 1 on error. If this happens, try more times
-      last unless ( $n->notify( \%info ) );
-    }
-  }
-  if ( $c->{notify}{script} ) {
-    my $bin = $c->{notify}{script};
-    if(-x $bin) {
-      open my $fh, '|-', $bin or die "Can't pipe to script >> $bin <<: $!";
-      say $fh jfreeze(\%info);
-      close $fh;
-    }
-  }
+  sendmail(%mail) or MSG($Mail::Sendmail::error);
 
-  return;
-}
-
-sub subject {
-  my $self = shift;
-  my $c    = $self->c;
-
-  return
-      '['
-    . localtime . ']['
-    . $c->{job_id} . ']['
-    . $self->log_status . '] '
-    . $c->{job_name} . ' ('
-    . $self->from_address() . ')';
-}
-
-sub log_status {
-  my ($self) = @_;
-
-  my $status;
-  if ( !$self->num_jobs || $self->num_jobs_failed == $self->num_jobs ) {
-    $status = 'ALL ERR';
-  } elsif ( $self->num_jobs_failed ) {
-    $status = $self->num_jobs_failed . '/' . $self->num_jobs . ' ERR';
-  } else {
-    $status = 'ALL OK';
-  }
-
-  return $status;
-}
-
-sub from_address {
-  return $ENV{SGE_O_LOGNAME} && $ENV{SGE_O_HOST}
-    ? join( '@', $ENV{SGE_O_LOGNAME}, $ENV{SGE_O_HOST} )
-    : join( '@', $ENV{USER}, ( $ENV{HOSTNAME} || hostname() ) );
-}
-
-sub gen_report {
-  my ($self, $full) = @_;
-  my @node_report = @{ $self->_node_log };
-  @node_report = ( @node_report[ 0 .. 13 ], '...' ) if (@node_report > 17  && !$full);
-  return join( "\n", $self->subject(), '', @{ $self->_log }, '', @node_report ) . "\n";
-}
-
-sub restart_script {
-  my ($self) = @_;
-  return join( "\n", @{ $self->_cmd_script } ) . "\n";
+  MSG( "Mail log says:\n", $Mail::Sendmail::log );
 }
 
 sub write {
   my ($self) = @_;
 
-  my $cmd_f = $self->failed_restart_file;
+  my $cmd_f = $self->error_cmd_file;
 
   open my $cmd_fh, '>', $cmd_f or confess "Can't open filehandle: $!";
-  print $cmd_fh $self->restart_script;
-  $cmd_fh->close;
+  print $cmd_fh join "\n", @{ $self->_cmd_script };
+  close $cmd_fh;
 
   chmod 0755, $cmd_f;
 
-  my $log_f = $self->failed_log_file;
+  my $log_f = $self->error_log_file;
 
   open my $log_fh, '>', $log_f or confess "Can't open filehandle: $!";
-
-  print $log_fh $self->gen_report(1);
-  $log_fh->close;
+  print $log_fh join "\n", @{ $self->_log_report };
+  close $log_fh;
 
   #CREATE SCRIPT TO UPDATE RERUN JOBS
   $self->_write_update_script;
@@ -402,11 +288,11 @@ sub _write_update_script {
   push @post_log_cmd, '-o', $c->{stdout_dir};
   push @post_log_cmd, @{ $c->{cmd} }, '--node_log', $c->{job_id}, $self->config_file;
 
-  my $update_log_file = $self->failed_update_file;
+  my $update_log_file = catfile( $c->{tmp_dir}, "update.$c->{job_name}.j$c->{job_id}.sh" );
 
   open my $update_log_fh, '>', $update_log_file or confess "Can't open filehandle: $!";
   print $update_log_fh join( " ", "cd", "'" . $c->{working_dir} . "'", '&&', @post_log_cmd ), "\n";
-  $update_log_fh->close;
+  close $update_log_fh;
 
   chmod 0755, $update_log_file;
 
